package com.caimei365.manager.controller.caimei.user;

import com.caimei365.manager.config.security.ConstantKey;
import com.caimei365.manager.config.security.JwtService;
import com.caimei365.manager.config.security.SecurityConfig;
import com.caimei365.manager.config.thinkgem.utils.Encodes;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.cmUser.*;
import com.caimei365.manager.service.caimei.user.CmClubRemarksService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/9/7
 */
@RestController
@RequestMapping("/user/remarks")
public class CmClubRemarksApi {

    @Autowired private CmClubRemarksService cmClubRemarksService;
    @Resource private JwtService jwtService;

    /**
     * 咨询记录类别
     * @return
     */
    @GetMapping("/getClassName")
    public ResponseJson<List<CmConsulttype>> getClassName() {
        return cmClubRemarksService.getClassName();
    }

    /**
     * 获取当天协销咨询记录信息
     * @return
     */
    @GetMapping("/getServiceList")
    public ResponseJson<Map<String, Object>> getServiceList(String startTime, String endTime) {
        return cmClubRemarksService.getServiceList(startTime, endTime);
    }

    /**
     * 已注册用户咨询记录
     *
     * @param cmClubRemarks
     * @return
     */
    @GetMapping("/registList")
    public ResponseJson<PaginationVo<CmClubRemarks>> registList(CmClubRemarks cmClubRemarks,
                                                 @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                 @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {

        return cmClubRemarksService.findRegistList(cmClubRemarks, pageNum, pageSize);
    }

    /**
     * 资料备注文件下载
     */
    @GetMapping("/remarks/download")
    public void downloadRemarks(String fileName, String ossName, HttpServletRequest request, HttpServletResponse response) throws IOException {

        cmClubRemarksService.downloadRemarks(fileName, ossName, request, response);
    }

    /**
     * 报备列表
     * @param cmReport
     * @param pageNum
     * @param pageSize
     * @return
     */
    @GetMapping("/ReportList")
    public ResponseJson<PaginationVo<CmReport>> getReportList(CmReport cmReport,
                                                              @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                              @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {

        return cmClubRemarksService.getReportList(cmReport, pageNum, pageSize);
    }

    /**
     * 报备列表查看详情
     * @param cmClubRemarks
     * @return
     */
    @GetMapping("/cmRegistReportList")
    public ResponseJson<List<CmClubRemarks>> cmRegistReportList(CmClubRemarks cmClubRemarks) {
        if (null == cmClubRemarks.getReportId()) {
            return ResponseJson.error(-1, "报备记录Id不能为空", null);
        }
        return cmClubRemarksService.cmRegistReportList(cmClubRemarks);
    }

    /**
     * 取消关联
     * @param remarksId
     * @param type
     * @return
     */
    @GetMapping("/audit")
    public ResponseJson audit(Integer remarksId, Integer type) {
        if (null == remarksId) {
            return ResponseJson.error(-1, "报备记录Id不能为空", null);
        }
        if (null == type) {
            return ResponseJson.error(-1, "报备记录类型不能为空", null);
        }
        return cmClubRemarksService.audit(remarksId, type);
    }

    /**
     * 删除报备列表
     * @param reportId
     * @return
     */
    @GetMapping("/deleteReport/{reportId}")
    public ResponseJson deleteReport(@PathVariable("reportId") Integer reportId) {
        if (null == reportId) {
            return ResponseJson.error(-1, "报备记录Id不能为空", null);
        }
        return cmClubRemarksService.deleteReport(reportId);
    }

    /**
     * 审核报备记录
     * @param reportId
     * @param auditText
     * @param status
     * @param request
     * @return
     */
    @GetMapping("/reportCount")
    public ResponseJson reportCount(Integer reportId, String auditText, Integer status, HttpServletRequest request) {
        String token = request.getHeader(ConstantKey.TOKEN_NAME);
        String userName = jwtService.getUsername(token);
        return cmClubRemarksService.reportCount(reportId, userName, auditText, status);
    }

    /**
     * 咨询记录类别
     * @param cmConsulttype
     * @param pageNum
     * @param pageSize
     * @return
     */
    @GetMapping("/getConsultList")
    public ResponseJson<PaginationVo<CmConsulttype>> getConsultList(CmConsulttype cmConsulttype,
                                                                    @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                    @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {
        return cmClubRemarksService.getConsultList(cmConsulttype, pageNum, pageSize);
    }

    /**
     * 保存咨询记录类别
     * @param cmConsulttype
     * @return
     */
    @PostMapping("/saveConsult")
    public ResponseJson saveConsult(@RequestBody CmConsulttype cmConsulttype) {

        return cmClubRemarksService.saveConsult(cmConsulttype);
    }


    /**
     * 删除咨询记录类别
     * @param id
     * @return
     */
    @GetMapping("/deleteConsult/{id}")
    public ResponseJson deleteConsult(@PathVariable("id") Integer id) {
        if (null == id) {
            return ResponseJson.error(-1, "咨询类别Id不能为空", null);
        }
        return cmClubRemarksService.deleteConsult(id);
    }

    /**
     * 排序
     * @param id
     * @param sortNumber
     * @return
     */
    @GetMapping("/sort")
    public ResponseJson sort(Integer id, Integer sortNumber) {
        if (null == id) {
            return ResponseJson.error(-1, "id不能为空", null);
        }
        if (null == sortNumber) {
            return ResponseJson.error(-1, "排序不能为空", null);
        }
        return cmClubRemarksService.sort(id, sortNumber);
    }

    /**
     * 未注册用户咨询记录
     * @param visit
     * @param pageNum
     * @param pageSize
     * @return
     */
    @GetMapping("/visitRemarkList")
    public ResponseJson<PaginationVo<VisitRemarkVo>> visitRemarkList(VisitRemarkVo visit,
                                                                     @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                     @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {

        return cmClubRemarksService.visitRemarkList(visit, pageNum, pageSize);
    }

    /**
     * 未注册用户数据详情
     */
    @GetMapping("/remarksList")
    public ResponseJson<PaginationVo<CmClubRemarks>> remarksList(VisitRemarkVo visitRemarkVo,
                              @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                              @RequestParam(value = "pageSize",defaultValue = "20") int pageSize) {

        return cmClubRemarksService.remarksList(visitRemarkVo, pageNum, pageSize);
    }

}
