package com.caimei365.manager.dao;

import com.caimei365.manager.entity.wechat.*;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/1/5
 */
@Mapper
public interface WeChatDao {
    /**
     * 获取微信公众号菜单列表
     * @param parentId 父级ID
     * @param wxType   微信公众号ID
     */
    List<WechatMenu> getWechatMenuList(Integer parentId, String wxType);
    /**
     * 根据ID获取微信公众号菜单
     */
    WechatMenu getWechatMenu(Integer id);
    /**
     * 根据ID更新微信公众号菜单
     */
    void updateWechatMenu(WechatMenu menu);
    /**
     * 添加微信公众号菜单
     */
    void insertWechatMenu(WechatMenu menu);
    /**
     * 根据ID删除微信公众号菜单
     */
    void deleteWechatMenu(Integer id);
    /**
     * 根据父级Id统计同级别菜单数量
     * @param parentId 父级ID
     */
    Integer countChildByParentId(Integer parentId,String wxType);

    /**
     * 获取微信公众号菜单列表
     * @param wxType 公众号Id
     * @param title 回复素材标题
     * @param responseType   事件类型
     * @param msgType   回复类型
     */
    List<WechatReply> getReplyList(String wxType, String title, String responseType, String msgType);
    /**
     * 根据ID获取回复配置
     */
    WechatReply getWechatReply(Integer id);
    /**
     * 根据ID更新回复配置
     */
    void updateWechatReply(WechatReply reply);
    /**
     * 添加回复配置
     */
    void insertWechatReply(WechatReply reply);
    /**
     * 根据ID删除微信公众号回复配置
     */
    void deleteWechatReply(Integer id);

    /**
     * 获取文本素材列表
     * @param wxType 公众号Id
     */
    List<WechatText> getTextList(String wxType, String title);
    /**
     * 根据ID获取文本素材
     */
    WechatText getWechatText(Integer id);
    /**
     * 根据ID更新文本素材
     */
    void updateWechatText(WechatText text);
    /**
     * 添加文本素材
     */
    void insertWechatText(WechatText text);
    /**
     * 根据ID删除文本素材
     */
    void deleteWechatText(Integer id);

    /**
     * 获取图文素材列表
     * @param wxType 公众号Id
     */
    List<WechatArticle> getArticleList(String wxType, String title);
    /**
     * 根据ID获取图文素材
     */
    WechatArticle getWechatArticle(Integer id);
    /**
     * 根据图文ID获取详情列表
     */
    List<WechatArticleDetail> getArticleDetailList(Integer articleId);
    /**
     * 根据图文ID获取详情Ids
     */
    List<Integer> getArticleDetailIds(Integer articleId);
    /**
     * 根据ID获取图文素材文章
     */
    WechatArticleDetail getWechatArticleDetail(Integer id);
    /**
     * 根据ID更新图文素材
     */
    void updateWechatArticle(WechatArticle article);
    /**
     * 更新图文素材文章
     */
    void updateWechatArticleDetail(WechatArticleDetail detail);
    /**
     * 添加图文素材
     */
    void insertWechatArticle(WechatArticle article);
    /**
     * 添加图文素材文章
     */
    void insertWechatArticleDetail(WechatArticleDetail detail);
    /**
     * 根据ID删除图文素材
     */
    void deleteWechatArticle(Integer id);
    /**
     * 根据ID删除图文素材文章列表
     */
    void deleteDetailByArticleId(Integer articleId);
    /**
     * 根据ID删除图文素材文章
     */
    void deleteWechatArticleDetail(Integer id);
}
