package com.caimei365.manager.dao.user;

import java.util.List;

import com.caimei365.manager.entity.caimei.cmUser.User;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

/**
 * 用户信息Mapper接口
 *
 * @author Kaick
 * @date 2023-12-14
 */
@Mapper
public interface UserMapper
{
    /**
     * 通过对象查询用户信息列表
     *
     * @param user 用户信息
     * @return 用户信息集合
     */
    List<User> getUserList(User user);

    /**
     * 通过Id查询用户信息对象
     *
     * @param userID 用户信息主键
     * @return 用户信息
     */
    User getUserById(String userID);

    /**
     * 通过对象查询用户信息对象
     *
     * @param user 用户信息
     * @return 用户信息
     */
    User getByUser(User user);

    /**
     * 通过对象查询用户信息Id
     *
     * @param user 用户信息
     * @return String
     */
    String getById(User user);


    /**
     * 通过对象查询用户信息记录总数
     *
     * @param user 用户信息
     * @return 用户信息Integer
     */
    int getCount(User user);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    int addUser(User user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    int updateUser(User user);

    /**
     * 删除用户信息
     *
     * @param userID 用户信息主键
     * @return 结果
     */
    int delUserByUserID(String userID);

    /**
     * 批量删除用户信息
     *
     * @param  需要删除的数据主键集合
     * @return 结果
     */
    int delUser(User user);

}
