package com.caimei365.manager.service.caimei.providers.impl;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.util.Date;
import java.util.List;
import java.util.NoSuchElementException;

import com.caimei365.manager.FastDFS.FastDFSClient;
import com.caimei365.manager.dao.CmRelatedImageMapper;
import com.caimei365.manager.dao.providers.CmProvidersContractMapper;
import com.caimei365.manager.dao.providers.CmProvidersMapper;
import com.caimei365.manager.entity.caimei.CmRelatedImage;
import com.caimei365.manager.entity.caimei.providers.CmProviders;
import com.caimei365.manager.entity.caimei.providers.CmProvidersContract;
import com.caimei365.manager.service.caimei.providers.CmProvidersContractService;
import com.caimei365.manager.service.caimei.providers.utils.ImageUtils;
import com.caimei365.manager.service.caimei.providers.utils.ProvidersTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import javax.imageio.ImageIO;

/**
 * Service业务层处理
 *
 * @author Kaick
 * @date 2023-12-12
 */
@Service
public class CmProvidersContractServiceImpl implements CmProvidersContractService {
    @Resource
    private CmProvidersContractMapper cmProvidersContractMapper;
    @Resource
    private CmProvidersMapper cmProvidersMapper;

    @Resource
    private CmRelatedImageMapper cmRelatedImageMapper;

    /**
     * 通过对象查询列表
     *
     * @param cmProvidersContract
     * @return
     */
    @Override
    public List<CmProvidersContract> getCmProvidersContractList(CmProvidersContract cmProvidersContract) {
        List<CmProvidersContract> cmProvidersContractList = cmProvidersContractMapper.getCmProvidersContractList(cmProvidersContract);
        return cmProvidersContractList;
    }

    /**
     * 通过Id查询
     *
     * @param id 主键
     * @return
     */
    @Override
    public CmProvidersContract getCmProvidersContractById(String id) {
        CmProvidersContract cmProvidersContractById = cmProvidersContractMapper.getCmProvidersContractById(id);
        cmProvidersContractById.setContractRelateds(cmRelatedImageMapper.getRelatedImageList(new CmRelatedImage().setType("4").setAuthorId(cmProvidersContractById.getId())));
        return cmProvidersContractById;
    }

    /**
     * 通过对象查询
     *
     * @param cmProvidersContract
     * @return
     */
    @Override
    public CmProvidersContract getByCmProvidersContract(CmProvidersContract cmProvidersContract) {
        CmProvidersContract byCmProvidersContract = cmProvidersContractMapper.getByCmProvidersContract(cmProvidersContract);
        byCmProvidersContract.setContractRelateds(cmRelatedImageMapper.getRelatedImageList(new CmRelatedImage().setType("4").setAuthorId(cmProvidersContract.getId())));
        return byCmProvidersContract;
    }


    /**
     * 新增
     *
     * @param cmProvidersContract
     * @return 结果
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int addCmProvidersContract(CmProvidersContract cmProvidersContract) {
        CmProviders cmProviders = cmProvidersMapper.getCmProvidersById(String.valueOf(cmProvidersContract.getProvidersId()));
        String levelName = cmProviders.getServiceLevel() == 1 ? "一级" : cmProviders.getServiceLevel() == 2 ? "二级" : "三级";
        BufferedImage image = null;
        try {
            image = ImageIO.read(new URL("https://img-b.caimei365.com/group1/M00/04/50/rB-lHGV6WsWABl00AARW2RH48Zg563.jpg"));
        } catch (IOException e) {
            e.printStackTrace();
        }
        BufferedImage realQrCodeImage = ImageUtils.createRealQrCode("https://www.caimei365.com/settlementInfo.html?id=" + cmProvidersContract.getProvidersId(), 1000);                //生成二维码
        cmProvidersMapper.updateCmProviders(new CmProviders()
                .setId(String.valueOf(cmProvidersContract.getProvidersId()))
                .setQrCodeImage(FastDFSClient.saveImage(realQrCodeImage))
                .setQualificationImage(ProvidersTemplate.setQualificationImage(image, cmProviders.getName(),    //生成资格证书
                        levelName,
                        cmProvidersContract.getContractStartDate(),
                        cmProvidersContract.getContractEndDate(),
                        cmProviders.getQualificationId(),
                        realQrCodeImage))
        );
        if(cmProvidersContract.getFlag()){
            if (null == cmProvidersContract.getId()) {
                cmProvidersContractMapper.addCmProvidersContract(cmProvidersContract); //新增服务商合同
                for (CmRelatedImage contractRelatedImage : cmProvidersContract.getContractRelateds()) {
                    cmRelatedImageMapper.addCmRelatedImage(new CmRelatedImage().setType("4").setAuthorId(cmProvidersContract.getId()).setImage(contractRelatedImage.getImage()).setFileName(contractRelatedImage.getFileName()));   //新增服务商合同文件
                }
            } else {
                cmProvidersContractMapper.updateCmProvidersContract(cmProvidersContract); //新增服务商合同
                CmRelatedImage cmRelatedImage = new CmRelatedImage().setType("4").setAuthorId(cmProvidersContract.getId());
                cmRelatedImageMapper.delCmRelatedImage(cmRelatedImage);
                for (CmRelatedImage contractRelatedImage : cmProvidersContract.getContractRelateds()) {
                    cmRelatedImageMapper.addCmRelatedImage(cmRelatedImage.setImage(contractRelatedImage.getImage()).setFileName(contractRelatedImage.getFileName()));     //新增服务商合同文件
                }
            }
        }
        return 1;
    }

    /**
     * 修改
     *
     * @param cmProvidersContract
     * @return 结果
     */
    @Override
    public int updateCmProvidersContract(CmProvidersContract cmProvidersContract) {
        cmProvidersContract.setUpdateTime(new Date());
        return cmProvidersContractMapper.updateCmProvidersContract(cmProvidersContract);
    }


}



