package com.caimei365.manager.service.caimei.providers.impl;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Date;
import java.util.List;
import java.util.NoSuchElementException;

import com.caimei365.manager.FastDFS.FastDFSClient;
import com.caimei365.manager.config.utils.DateUtil;
import com.caimei365.manager.dao.CmRelatedImageMapper;
import com.caimei365.manager.dao.providers.CmProvidersContractMapper;
import com.caimei365.manager.dao.providers.CmProvidersMapper;
import com.caimei365.manager.dao.user.UserMapper;
import com.caimei365.manager.entity.caimei.CmRelatedImage;
import com.caimei365.manager.entity.caimei.cmUser.User;
import com.caimei365.manager.entity.caimei.providers.CmProviders;
import com.caimei365.manager.entity.caimei.providers.CmProvidersContract;
import com.caimei365.manager.entity.po.UploadFilePo;
import com.caimei365.manager.service.caimei.providers.CmProvidersContractService;
import com.caimei365.manager.service.caimei.providers.CmProvidersService;
import com.caimei365.manager.service.caimei.providers.utils.ImageUtils;
import com.caimei365.manager.service.caimei.providers.utils.ProvidersTemplate;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Service业务层处理
 *
 * @author Kaick
 * @date 2023-12-12
 */
@Slf4j
@Service
public class CmProvidersServiceImpl implements CmProvidersService {
    @Resource
    private CmProvidersMapper cmProvidersMapper;

    @Resource
    private CmProvidersContractMapper cmProvidersContractMapper;
    @Resource
    private CmProvidersContractService cmProvidersContractService;

    @Resource
    private CmRelatedImageMapper cmRelatedImageMapper;
    @Resource
    private UserMapper userMapper;


    /**
     * 通过对象查询列表
     *
     * @param cmProviders
     * @return
     */
    @Override
    public List<CmProviders> getCmProvidersList(CmProviders cmProviders) {
        return cmProvidersMapper.getCmProvidersList(cmProviders);
    }

    /**
     * 通过Id查询
     *
     * @param id 主键
     * @return
     */
    @Override
    public CmProviders getCmProvidersById(String id) {
        CmProviders cmProvidersById = cmProvidersMapper.getCmProvidersById(id);
        cmProvidersById.setTextRelateds(cmRelatedImageMapper.getRelatedImageList(new CmRelatedImage().setType("2").setAuthorId(cmProvidersById.getId())));
        cmProvidersById.setInfoRelateds(cmRelatedImageMapper.getRelatedImageList(new CmRelatedImage().setType("3").setAuthorId(cmProvidersById.getId())));
        return cmProvidersById;
    }

    /**
     * 通过对象查询
     *
     * @param cmProviders
     * @return
     */
    @Override
    public CmProviders getByCmProviders(CmProviders cmProviders) {
        CmProviders providers = cmProvidersMapper.getByCmProviders(cmProviders);
        providers.setTextRelateds(cmRelatedImageMapper.getRelatedImageList(new CmRelatedImage().setType("2").setAuthorId(cmProviders.getId())));
        providers.setInfoRelateds(cmRelatedImageMapper.getRelatedImageList(new CmRelatedImage().setType("3").setAuthorId(cmProviders.getId())));
        return providers;
    }


    /**
     * 新增
     *
     * @param cmProviders
     * @return 结果
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int addCmProviders(CmProviders cmProviders) {
        if (StringUtils.isBlank(cmProviders.getId())) {
            User user = new User();
            user.setBindMobile(cmProviders.getMobile());
            user.setUserName(cmProviders.getLinkMan());
            user.setName(cmProviders.getName());
            user.setUserIdentity(9);
            user.setRegisterUserTypeID("-1");
            user.setRegisterTime(DateUtil.getDateTime());
            user.setValidFlag("1");
            userMapper.addUser(user);
            cmProviders.setUserId(Integer.valueOf(user.getUserID()));
            String qualificationId = "CMFW0" + cmProviders.getServiceLevel();
            if (0 == cmProvidersMapper.getCount(null)) {
                qualificationId = qualificationId + "000001";
            } else {
                int num = Integer.parseInt(cmProvidersMapper.getByCmProviders(null).getQualificationId().substring(6));
                num++; // 每次加1
                // 使用字符串格式化保持位数不变，总共有6位，使用0进行填充
                qualificationId = qualificationId + String.format("%06d", num);
            }
            cmProviders.setQualificationId(qualificationId);
            cmProviders.setStatus(90);
            cmProviders.setCreateTime(new Date());
            cmProvidersMapper.addCmProviders(cmProviders);   //新增服务商
            String providersId = cmProviders.getId();
            cmProviders.getContract().setProvidersId(Integer.valueOf(providersId));
            cmProviders.getContract().setCreateBy(cmProviders.getCreateBy());
            cmProvidersContractService.addCmProvidersContract(cmProviders.getContract()); //新增服务商合同
            for (CmRelatedImage textRelatedImage : cmProviders.getTextRelateds()) {
                cmRelatedImageMapper.addCmRelatedImage(new CmRelatedImage().setType("2").setAuthorId(providersId).setImage(textRelatedImage.getImage()).setFileName(textRelatedImage.getFileName()));      //新增服务商简介图
            }
            for (CmRelatedImage infoRelatedImage : cmProviders.getInfoRelateds()) {
                cmRelatedImageMapper.addCmRelatedImage(new CmRelatedImage().setType("3").setAuthorId(providersId).setImage(infoRelatedImage.getImage()).setFileName(infoRelatedImage.getFileName()));      //新增服务资料文件
            }
            return 1;
        } else {
            User user = new User();
            user.setBindMobile(cmProviders.getMobile());
            user.setUserName(cmProviders.getLinkMan());
            user.setName(cmProviders.getName());
            user.setValidFlag(cmProviders.getStatus() == 90 ? "1" : "0");
            userMapper.updateUser(user);
            String qualificationId = new StringBuffer(cmProviders.getQualificationId()).replace(5, 6, cmProviders.getServiceLevel().toString()).toString();
            cmProviders.setQualificationId(qualificationId);
            CmRelatedImage cmRelatedImage = new CmRelatedImage().setType("3").setAuthorId(cmProviders.getId());
            cmRelatedImageMapper.delCmRelatedImage(cmRelatedImage);      //删除旧服务资料文件
            cmRelatedImage.setType("2");
            cmRelatedImageMapper.delCmRelatedImage(cmRelatedImage);      //删除旧服务商简介图
            for (CmRelatedImage textRelatedImage : cmProviders.getTextRelateds()) {
                cmRelatedImageMapper.addCmRelatedImage(cmRelatedImage.setImage(textRelatedImage.getImage()).setFileName(textRelatedImage.getFileName()));      //新增服务商简介图
            }
            cmRelatedImage.setType("3");
            for (CmRelatedImage infoRelatedImage : cmProviders.getInfoRelateds()) {
                cmRelatedImageMapper.addCmRelatedImage(cmRelatedImage.setImage(infoRelatedImage.getImage()).setFileName(infoRelatedImage.getFileName()));       //新增服务资料文件
            }
            cmProvidersMapper.updateCmProviders(cmProviders);
            CmProvidersContract contract = cmProvidersContractMapper.getByCmProvidersContract(new CmProvidersContract().setProvidersId(Integer.valueOf(cmProviders.getId())));
            contract.setFlag(false);//更改新增标识
            cmProvidersContractService.addCmProvidersContract(contract); //新增服务商合同
            return 1;
        }
    }

    /**
     * 修改
     *
     * @param cmProviders
     * @return 结果
     */
    @Override
    public int updateCmProviders(CmProviders cmProviders) {
        if (null != cmProviders.getStatus()) {
            userMapper.updateUser(new User()
                    .setUserID(String.valueOf(cmProviders.getUserId()))
                    .setValidFlag(cmProviders.getStatus() == 90 ? "1" : "0")
            );
        }
        cmProviders.setUpdateTime(new Date());
        return cmProvidersMapper.updateCmProviders(cmProviders);
    }


    /**
     * 下載圖片
     */
    @Override
    public void downloadImage(String ids, Integer type, HttpServletRequest request, HttpServletResponse response) throws Exception {
        // 将图片压缩成zip文件
        List<UploadFilePo> uploadFileList = cmProvidersMapper.getProvidersUploadFileList(ids, type);
        String imageZipPath = ImageUtils.createImageZip(uploadFileList, type);
        FastDFSClient.download(request, response, imageZipPath, imageZipPath.substring(imageZipPath.lastIndexOf("/")));
        // 删除临时文件夹
        String tempPath = imageZipPath.substring(0, imageZipPath.lastIndexOf("/"));
        log.info("【图片压缩包】,删除图片压缩包文件夹:" + ImageUtils.deleteFile( new File(tempPath)));
    }

}



