package com.caimei365.manager.service.caimei.providers.utils;

import com.caimei365.manager.FastDFS.FastDFSClient;
import com.caimei365.manager.entity.po.UploadFilePo;
import com.caimei365.manager.utils.FileIOUtils;
import com.github.tobato.fastdfs.service.FastFileStorageClient;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.List;

/**
 * 生成二维码工具类.
 *
 * @author linzp
 * @version 1.0.0
 * CreateDate 2021/1/26 10:39
 */
@Slf4j
@Component
public class ImageUtils {

    private static String config;

    private static String imageDomain;


    @Value("${cm.config}")
    public void setConfig(String config) {
        ImageUtils.config = config;
    }

    @Value("${caimei.imageDomain}")
    public void setImageDomain(String imageDomain) {
        ImageUtils.imageDomain = imageDomain;
    }

    /**
     * BufferImage转byte[]
     *
     * @param original
     * @return
     */
    public static byte[] bufImg2Bytes(BufferedImage original) {
        ByteArrayOutputStream bStream = new ByteArrayOutputStream();
        try {
            ImageIO.write(original, "png", bStream);
        } catch (IOException e) {
            throw new RuntimeException("bugImg读取失败:" + e.getMessage(), e);
        }
        return bStream.toByteArray();
    }

    /**
     * byte[]转BufferImage
     * @param imgBytes
     * @return
     */
    public static BufferedImage bytes2bufImg(byte[] imgBytes){
        BufferedImage tagImg = null;
        try {
            tagImg = ImageIO.read(new ByteArrayInputStream(imgBytes));
            return tagImg;
        } catch (IOException e) {
            throw new RuntimeException("bugImg写入失败:"+e.getMessage(),e);
        }
    }

    public static String imageTemplate(BufferedImage image, List<Map<String,Object>> imageList) {
        try (ByteArrayOutputStream os = new ByteArrayOutputStream()){
            // 合成图片
            BufferedImage result = image;
            // 将两张图片放入结果图片中
            Graphics2D g2d = result.createGraphics();
            g2d.drawImage(result, 0, 0, null);
            for (Map<String,Object> imageMap : imageList) {
                int type = (int) imageMap.get("type");
                int x = (int) imageMap.get("x");
                int y = (int) imageMap.get("y");
                if (1 == type) {
                    BufferedImage imageFile = (BufferedImage) imageMap.get("image");
                    if(null!=imageMap.get("align")){
                        x=x-imageFile.getWidth()/2;
                        // y=y-imageFile.getHeight()/2;
                    }
                    g2d.drawImage(imageFile, x, y, null);
                } else {
                    String text = (String) imageMap.get("text");
                    //设置字体
                    g2d.setFont(null==imageMap.get("font")? new Font("思源宋体", Font.PLAIN, 32):(Font) imageMap.get("font"));
                    //设置颜色
                    g2d.setColor(null==imageMap.get("color")?Color.BLACK:(Color) imageMap.get("color"));
                    if(null!=imageMap.get("align")){
                        int width = g2d.getFontMetrics().stringWidth(text);
                        x=x-width/2;
                        // y=y-g2d.getFont().getSize()/2;
                    }
                    //添加文字
                    g2d.drawString(text, x, y);
                }
            }
            // 保存合成图片
            ImageIO.write(result, "jpg", os);
            // 释放资源
            g2d.dispose();
            return FastDFSClient.saveFile(new ByteArrayInputStream(os.toByteArray()));
        } catch (Exception e) {
            e.printStackTrace();
            log.error("生成图片异常!");
            return null;
        }
    }


    public static BufferedImage createRealQrCode(String url,int qrCodeSize) {
        int BLACK = 0xFF000000;
        int WHITE = 0xFFFFFFFF;
        try {
            Map<EncodeHintType, String> hints = new HashMap<>();
            hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");
            //设置二维码白边的大小
            hints.put(EncodeHintType.MARGIN, "0");
            BitMatrix bitMatrix = new MultiFormatWriter().encode(url, BarcodeFormat.QR_CODE, qrCodeSize, qrCodeSize, hints);
            //自定义白边边框宽度
            int margin = 1;
            bitMatrix = updateBit(bitMatrix, margin);
            int width = bitMatrix.getWidth();
            int height = bitMatrix.getHeight();
            BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
            for (int x = 0; x < width; x++) {
                for (int y = 0; y < height; y++) {
                    image.setRGB(x, y, bitMatrix.get(x, y) ? BLACK : WHITE);
                }
            }
            //根据size放大、缩小生成的二维码
            image = zoomInImage(image,qrCodeSize,qrCodeSize);
            return image;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private static BitMatrix updateBit(BitMatrix matrix, int margin){
        int tempM = margin*2;
        //获取二维码图案的属性
        int[] rec = matrix.getEnclosingRectangle();
        int resWidth = rec[2] + tempM;
        int resHeight = rec[3] + tempM;
        // 按照自定义边框生成新的BitMatrix
        BitMatrix resMatrix = new BitMatrix(resWidth, resHeight);
        resMatrix.clear();
        //循环，将二维码图案绘制到新的bitMatrix中
        for(int i= margin; i < resWidth- margin; i++){
            for(int j=margin; j < resHeight-margin; j++){
                if(matrix.get(i-margin + rec[0], j-margin + rec[1])){
                    resMatrix.set(i,j);
                }
            }
        }
        return resMatrix;
    }

    public static BufferedImage zoomInImage(BufferedImage  originalImage, int width, int height){
        BufferedImage newImage = new BufferedImage(width,height,originalImage.getType());
        Graphics g = newImage.getGraphics();
        g.drawImage(originalImage, 0,0,width,height,null);
        g.dispose();
        return newImage;
    }

    /**
     * 创建图片压缩包
     *
     * @param type    1服务商资格证书，2服务商二维码
     * @return
     * @throws Exception
     */
    public static String createImageZip(List<UploadFilePo> uploadFilePos, Integer type) throws Exception {
        String randomStr = UUID.randomUUID().toString();
        // 图片暂存本地服务器路径
        String filePath = "/mnt/newdatadrive/data/runtime/jar-instance/manager-api/tempImage/";
        if ("dev".equals(config)) {
            filePath = "D:\\uploadZip\\";
        }
        // 压缩文件夹上层临时文件夹路径
        String tempPath = filePath + randomStr;
        // 压缩文件夹路径
        String basePath = "";
        // 生成唯一临时文件夹
        filePath += randomStr + "/";
        // 在临时文件夹中生成要压缩的基础文件夹
        String baseDirName = "";
        switch (type) {
            case 1:
                baseDirName = "服务商资格证书";
                break;
            case 2:
                baseDirName = "服务商二维码";
                break;
            default:
                baseDirName = "";
        }
        filePath += baseDirName+ "/" ;
        basePath = filePath;
        File baseDir = new File(basePath);
        if (!baseDir.exists()) {
            baseDir.mkdirs();
        }
        // 往基础文件夹中填充相应数据
        for (int i = 0; i < uploadFilePos.size(); i++) {
            // 把图片链接转换为图片
            UploadFilePo file = uploadFilePos.get(i);
            String fileUrl = file.getFileUrl();
            URL url = new URL(fileUrl);
            InputStream inputStream = url.openStream();
            BufferedImage image = ImageIO.read(inputStream);
            String extName = fileUrl.substring(fileUrl.lastIndexOf(".") + 1);
            File buildFile = buildFileName(filePath ,file.getFileName(),"."+extName,0);
            ImageIO.write(image, extName, buildFile);
            inputStream.close();
        }
        // 将文件夹转为压缩文件
        String zipPath = tempPath + "/" + baseDirName + ".zip";
        FileOutputStream zipOut = new FileOutputStream(zipPath);
        FileIOUtils.zipFolder(basePath, zipOut, true);
        return zipPath;
    }


    public static File buildFileName(String path, String fileName, String suffix, Integer index) {
        File file;
        //下标不等于0开始拼后缀
        if (index != 0) {
            file = new File(path + fileName + "(" + index + ")" + suffix);
        } else {
            file = new File(path + fileName + suffix);
        }
        //判断文件是否存在 文件不存在退出递归
        if (file.isFile()) {
            //每次递归给下标加1
            file = buildFileName(path, fileName, suffix, ++index);
        }
        return file;
    }

    public static boolean deleteFile(File file){
        //判断文件不为null或文件目录存在
        if (file == null || !file.exists()){
            log.error("文件删除失败,请检查文件路径是否正确>>>"+file.getAbsolutePath());
            return false;
        }
        //取得这个目录下的所有子文件对象
        File[] files = file.listFiles();
        //遍历该目录下的文件对象
        for (File f: files){
            //判断子目录是否存在子目录,如果是文件则删除
            if (f.isDirectory()){
                deleteFile(f);
            }else {
                f.delete();
            }
        }
        //删除空文件夹  for循环已经把上一层节点的目录清空。
        return file.delete();
    }
}
