package com.caimei365.manager.utils;

import lombok.extern.slf4j.Slf4j;
import net.lingala.zip4j.core.ZipFile;
import net.sf.sevenzipjbinding.ArchiveFormat;
import net.sf.sevenzipjbinding.IInArchive;
import net.sf.sevenzipjbinding.SevenZip;
import net.sf.sevenzipjbinding.SevenZipException;
import net.sf.sevenzipjbinding.impl.RandomAccessFileInStream;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.IOUtils;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.*;
import java.nio.charset.Charset;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * Description
 *
 * @author : hzg
 * @date : 2023/12/19
 */
@Slf4j
public class FileUtil {

    /**
     * 将file 文件转为 MultipartFile 文件
     * @param file
     * @return
     * @throws IOException
     */
    public static MultipartFile fileToMultipartFileConverter(File file) throws IOException {
        DiskFileItem diskFileItem = new DiskFileItem("File", "application/octet-stream", true, file.getName(), (int) file.length(), file.getParentFile());
        try(InputStream is = new FileInputStream(file)) {
            diskFileItem.getOutputStream().write(IOUtils.toByteArray(is));
        }
        return new CommonsMultipartFile(diskFileItem);
    }



    /**
     * zip文件解压
     *
     * @param destPath 解压文件路径
     * @param zipFilePath  压缩文件路径
     */
    public static void unZip(String zipFilePath, String destPath) throws IOException {
        File destDir = new File(destPath);
        if (!destDir.exists()) {
            destDir.mkdir();
        }
        ZipInputStream zipIn = new ZipInputStream(new FileInputStream(zipFilePath), Charset.forName("GBK"));
        ZipEntry entry = zipIn.getNextEntry();
        while (entry != null) {
            String filePath = destPath + File.separator + entry.getName();
            if (!entry.isDirectory()) {
                File file = new File(filePath);
                file.getParentFile().mkdirs();
                try (FileOutputStream fos = new FileOutputStream(file)) {
                    byte[] bytes = new byte[4096];
                    int length;
                    while ((length = zipIn.read(bytes)) != -1) {
                        fos.write(bytes, 0, length);
                    }
                }
            } else {
                File dir = new File(filePath);
                dir.mkdir();
            }
            zipIn.closeEntry();
            entry = zipIn.getNextEntry();
        }
        zipIn.close();
    }

    /**
     * zip文件解压
     *
     * @param destPath 解压文件路径
     * @param zipFile  压缩文件
     */
    public static void unPackZip(File zipFile, String destPath) {
        try {
            ZipFile zip = new ZipFile(zipFile);
            /*zip4j默认用GBK编码去解压,这里设置编码为GBK的*/
            zip.setFileNameCharset("GBK");
            log.info("begin unpack zip file....");
            zip.extractAll(destPath);
        } catch (Exception e) {
            log.error("解压失败：", e.getMessage(), e);
        }
    }

    /**
     * rar文件解压
     *
     * @param rarFile  rar压缩包
     * @param rarFile  解压保存路径下属名称
     * @param destPath 解压保存路径
     */
    public static void unPackRar(File rarFile, String filename, String destPath) {
        try {
            // f -  压缩文件
            RandomAccessFile randomAccessFile = new RandomAccessFile(rarFile.getAbsolutePath(), "r");
            IInArchive archive = SevenZip.openInArchive(ArchiveFormat.RAR5,  new RandomAccessFileInStream(randomAccessFile));
            // 解压文件路径
            File zdir = new File(destPath);
            if (!zdir.exists() || !zdir.isDirectory()) {
                zdir.mkdir();
            }

            int[] in = new int[archive.getNumberOfItems()];
            for(int i=0;i<in.length;i++){
                in[i] = i;
            }
            archive.extract(in, false, new ExtractCallback(archive, zdir.getAbsolutePath()));
            archive.close();
            randomAccessFile.close();
        } catch (FileNotFoundException | SevenZipException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
